package ClamTk::Prefs;

use strict;
#use warnings; # disabled upon release

use Carp qw(cluck);
$|++;

use encoding 'utf8';

use File::Path;
use Locale::gettext;
use POSIX qw/locale_h/;
textdomain('clamtk');
setlocale( LC_MESSAGES, '' );
bind_textdomain_codeset( 'clamtk', 'UTF-8' );

sub start_up {
    if ( !-d $main::scanner->{viruses} ) {
        eval { mkpath( $main::scanner->{viruses}, 0, oct(755) ); };
        return 0 if ($@);
    } else {
        chmod oct(755), $main::scanner->{viruses};
    }
    if ( !-d $main::scanner->{history} ) {
        eval { mkpath( $main::scanner->{history}, 0, oct(755) ); };
        return 0 if ($@);
    } else {
        chmod oct(755), $main::scanner->{history};
    }
    if ( !-d $main::scanner->{db} ) {
        eval { mkpath( $main::scanner->{db}, 0, oct(755) ); };
        cluck $@  if $@;
        return 0 if ($@);
    } else {
        chmod oct(755), $main::scanner->{db};
    }
    if ( !-e $main::scanner->{prefs} ) {
        open( my $F, '>', $main::scanner->{prefs} )
            or die "Unable to create preferences! $!\n";
        close($F);
        eval { custom_prefs() };
        return 0 if ($@);
    }
    if ( !-e $main::scanner->{restore} ) {
        open( my $F, '>', $main::scanner->{restore} )
            or die "Unable to create restore file! $!\n";
        close($F);
    }

    return 1;
}

sub custom_prefs {

   # ensure prefs have normalized variables, especially for 3.11 -> 4.00 users
    open( my $F, '<', $main::scanner->{prefs} )
        or die "Unable to read preferences! $!\n";

    my %pkg;

    while (<$F>) {
        my ( $k, $v ) = split(/=/);
        chomp($v);
        $pkg{$k} = $v;
    }

    if ( !exists $pkg{Update} ) {
        $pkg{Update} = 'undef';
    } elsif ( $pkg{Update} !~ /shared|single/ ) {
        $pkg{Update} = 'undef';
    }

    for my $n (qw/LastScan LastInfection/) {
        if ( !exists $pkg{$n} ) {
            $pkg{$n} = gettext('Never');
        }
    }

    for my $o (
        qw/SaveToLog ScanHidden SizeLimit
        Thorough Recursive/
        )
    {
        if ( !exists $pkg{$o} ) {
            $pkg{$o} = 0;    # off by default
        }
    }

    for my $p (qw/OpenDNS AVCheck GUICheck/) {
        if ( !exists $pkg{$p} ) {
            $pkg{$p} = 1;    # on by default
        }
    }
    write_all(%pkg);
}

sub get_all_prefs {
    open( my $F, '<', $main::scanner->{prefs} )
        or die "Unable to read preferences! $!\n";

    my %pkg;

    while (<$F>) {
        my ( $k, $v ) = split(/=/);
        chomp($v);
        $pkg{$k} = $v;
    }
    return %pkg if %pkg;
}

sub write_all {
    my %loc = @_;
    open( my $F, '>', $main::scanner->{prefs} )
        or die "Unable to write preferences! $!\n";

    while ( my ( $k, $v ) = each %loc ) {
        print $F "$k=$v\n";
    }
    close($F);

    return 1;
}

sub set {
    my $self = shift;
    my ( $wk, $wv ) = @_;

    open( my $F, '<', $main::scanner->{prefs} )
        or die "Unable to read preferences! $!\n";

    my %pkg;
    while (<$F>) {
        my ( $k, $v ) = split(/=/);
        chomp($v);
        $pkg{$k} = $v;
    }
    close($F);

    open( $F, '>', $main::scanner->{prefs} )
        or die "Unable to write preferences! $!\n";

    while ( my ( $k, $v ) = each %pkg ) {
        print $F "$k=$v\n" unless ( $k eq $wk );
    }
    print $F "$wk=$wv\n";
    close($F);
}

sub get_property {
    my $self = shift;
    my ($wanted) = shift;

    open( my $F, '<', $main::scanner->{prefs} )
        or die "Unable to read preferences! $!\n";

    my %pkg;

    while (<$F>) {
        my ( $k, $v ) = split(/=/);
        chomp($v);
        $pkg{$k} = $v;
    }

    return unless %pkg;
    return $pkg{$wanted} || '';
}

sub restore {
    shift;
    my $wanted = shift;
    my $job    = shift;
    my $path   = shift;
    my $perm   = shift;

    my %p;

    open( my $F, '<', $main::scanner->{restore} ) or do {
        warn "Can't open restore file for reading: $!\n";
        return;
    };

    while (<$F>) {
        my ( $m, $path, $perm ) = split /:/;
        if ( exists $p{$m} ) {
            #warn "File $m already exists?\n";
            return;
        }
        $p{$m} = { path => $path, perm => $perm };
    }
    close($F);

    if ( $job eq 'exists' ) {
        for my $e ( keys %p ) {
            if ( $e eq $wanted ) {
                return ( $p{$e}->{path}, $p{$e}->{perm} );
            }
        }
        return 0;
    }

    if ( $job eq 'add' ) {
        if ( exists( $p{$wanted} ) ) {
            #warn "File $wanted already exists?\n";
            return;
        }
        open( $F, '>', $main::scanner->{restore} ) or do {
            warn "Can't open restore file for writing: $!\n";
            return;
        };

        if ( scalar( keys %p ) ) {
            for my $e ( keys %p ) {
                print $F $e, ":", $p{$e}->{path}, ":", $p{$e}->{perm};
            }
        }
        print $F "$wanted:$path:$perm\n";
        close($F);
    }

    if ( $job eq 'remove' ) {
        open( $F, '>', $main::scanner->{restore} ) or do {
            warn "Can't open restore file for writing: $!\n";
            return;
        };

        for my $e ( keys %p ) {
            next if ( $e eq $wanted );
            print $F $e, ":", $p{$e}->{path}, ":", $p{$e}->{perm};
        }
        close($F);
    }
}

1;
